"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var core_1 = require("../utils/core");
var regexp_1 = require("../utils/regexp");
var strings_1 = require("../utils/strings");
var svg_1 = require("../utils/svg");
var angular_templates_1 = require("./angular.templates");
var typescript_1 = require("./typescript");
function render(svgFile, options) {
    delete svgFile.contextInterface;
    return angular_templates_1.angularDynamicClassTemplate({
        className: strings_1.pascalCase(svgFile.name),
        selector: strings_1.kebabCase(options.module) + "-" + strings_1.kebabCase(svgFile.name)
    });
}
function getSvgAOT(svgFile, options) {
    var svgTemplate = ("\n      <fvg\n        [attr.class]=\"\\'" + strings_1.kebabCase(options.module) + "-\\'+context.uuid\"\n        [attr.width]=\"width\"\n        [attr.height]=\"height\"\n        [attr.viewBox]=\"viewBox\">\n        @@@styles@@@" + svgFile.svg + "\n      </fvg>")
        .replace(/ (\S+?)=['"]{{(.+?)}}['"]/g, " [attr.$1]=\"context.$2\"")
        .replace(/{{(.+?)}}/g, '{{context.$1}}')
        .replace('@@@styles@@@', !svgFile.css ? '' : "<style>" + svgFile.css.replace(/{{(.+?)}}/g, '{{context.$1}}') + "</style>");
    svgTemplate = replaceIds(svgTemplate)
        .replace(/xlink:href=["']#(.*?)(-{{context.uuid}})["']/g, "[attr.xlink:href]=\"getXlinkBase(\\'$1\\')\"")
        .replace(/([\S]*?)=["']url\(#(.*?)(-{{context.uuid}})\)["']/g, "[attr.$1]=\"getURLBase(\\'$2\\')\"")
        .replace(/:[ ]?url\((#.*?)(-{{context.uuid}})\)/g, ":{{getURLBase(\\'$1\\')}}");
    return svg_1.compactSVG(svgTemplate.replace(/\s/g, ' '))
        .replace('<fvg', '<svg')
        .replace('</fvg>', '</svg>');
}
function getStaticSvgAOT(svgFile, options) {
    var svgTemplate = ("\n      <fvg>@@@styles@@@" + svgFile.svg + "</fvg>")
        .replace(/ (\S+?)=['"]{{(.+?)}}['"]/g, " [attr.$1]=\"context.$2\"")
        .replace(/{{(.+?)}}/g, '{{context.$1}}')
        .replace('@@@styles@@@', !svgFile.css ? '' : "<style>" + svgFile.css.replace(/{{(.+?)}}/g, '{{context.$1}}') + "</style>");
    return svg_1.compactSVG(svgTemplate.replace(/\s/g, ' '))
        .replace('<fvg>', '')
        .replace('</fvg>', '');
}
function saveFile(options, blueprint) {
    var output = options.output, module = options.module;
    return function (svgFile) {
        // TS
        var filePath = "" + output + path.sep + module + path.sep + "assets" + path.sep + svgFile.name + ".ts";
        var destBase = path.dirname(filePath);
        if (!fs.existsSync(destBase)) {
            core_1.mkdirRecursiveSync(destBase);
        }
        delete svgFile.path;
        svgFile.svg = svgFile.contextDefaults ? getSvgAOT(svgFile, options) : getStaticSvgAOT(svgFile, options);
        delete svgFile.css;
        fs.writeFileSync(filePath, typescript_1.render(svgFile, options));
        if (svgFile.contextDefaults) {
            var ngFilePath = "" + output + path.sep + module + path.sep + "components" + path.sep + svgFile.name + ".component.ts";
            var destBase_1 = path.dirname(ngFilePath);
            if (!fs.existsSync(destBase_1)) {
                core_1.mkdirRecursiveSync(destBase_1);
            }
            fs.writeFileSync(ngFilePath, render(svgFile, options));
        }
    };
}
exports.saveFile = saveFile;
function generateIndexFile(options, files) {
    // generate an index.ts of svg's
    var indexFileExports = files
        .map(function (file) {
        var svgObjectName = strings_1.pascalCase(file.name);
        var context = file.contextDefaults ? ", " + svgObjectName + "Context" : '';
        return "export { " + svgObjectName + context + " } from './" + file.name + "';";
    })
        .join('\n');
    var indexFileType = "export type " + strings_1.pascalCase(options.module) + "Asset = " + files
        .map(function (file) {
        return "'" + file.name + "'";
    })
        .join('|') + ";";
    var indexFile = indexFileExports.concat(indexFileType).concat("\n    export function getSVGViewBox(viewBox: any): string {\n      return [viewBox.minx, viewBox.miny, viewBox.width, viewBox.height].join(' ');\n    }\n  ");
    fs.writeFileSync("" + options.output + path.sep + options.module + path.sep + "assets" + path.sep + "index.ts", indexFile);
    var components = files.filter(function (file) { return file.contextDefaults; });
    var namedComponents = components.map(function (_) { return strings_1.pascalCase(_.name); });
    var moduleName = strings_1.pascalCase(options.module);
    var componentsIndex = components
        .map(function (component) {
        return "export { " + strings_1.pascalCase(component.name) + "Component } from './" + component.name + ".component';";
    })
        .concat("export {" + strings_1.pascalCase(moduleName) + "Component} from './" + options.module + ".component'")
        .join('\n');
    var componentsIndexDir = "" + options.output + path.sep + options.module + path.sep + "components";
    if (!fs.existsSync(componentsIndexDir)) {
        core_1.mkdirRecursiveSync(componentsIndexDir);
    }
    // generate an index.ts of components
    fs.writeFileSync("" + componentsIndexDir + path.sep + "index.ts", componentsIndex);
    // generate the module.ts file
    fs.writeFileSync("" + options.output + path.sep + options.module + path.sep + strings_1.kebabCase(options.module) + ".module.ts", angular_templates_1.angularDynamicModuleTemplate({
        components: namedComponents.concat(strings_1.pascalCase(moduleName)),
        moduleName: moduleName
    }));
    fs.writeFileSync("" + options.output + path.sep + options.module + path.sep + "components" + path.sep + strings_1.kebabCase(options.module) + ".component.ts", angular_templates_1.svgIconClassTemplate({
        assets: files.map(function (file) { return strings_1.pascalCase(file.name); }),
        components: components.map(function (component) { return ({
            component: strings_1.pascalCase(component.name),
            name: component.name
        }); }),
        moduleName: moduleName,
        className: "" + strings_1.pascalCase(options.module),
        pascalCase: strings_1.pascalCase,
        selector: "" + strings_1.kebabCase(options.module)
    }));
}
exports.generateIndexFile = generateIndexFile;
function replaceIds(source) {
    var seed = '-{{context.uuid}}';
    var ids = [];
    var foundIds = source.match(/id="(.*?)"/g);
    if (foundIds) {
        ids = foundIds.map(function (m) {
            var a = m.match(/id="(.*?)"/);
            if (a) {
                return a[1];
            }
            return '';
        });
    }
    var prefixed = "svg2ts-";
    // content
    var result = ids.reduce(function (acc, id) {
        acc = acc
            // prefix document id's
            .replace(new RegExp('["\']' + id + '["\']', 'g'), "\"" + prefixed + id + seed + "\"")
            // replace document id refs
            .replace(new RegExp('["\']#' + id + '["\']', 'g'), "\"#" + prefixed + id + seed + "\"")
            // replace document id refs in url's
            .replace(new RegExp('url\\(#' + id + '\\)', 'g'), "url(#" + prefixed + id + seed + ")");
        return acc;
    }, source);
    // styles
    var styles = result.match(regexp_1.styleExtractRegExp);
    if (styles) {
        result = styles.reduce(function (acc, styleDef) {
            var styleDefSeedIds = ids.reduce(function (acc, id) {
                acc = acc.replace(new RegExp('#' + id + '', 'g'), "#" + prefixed + id + seed);
                return acc;
            }, styleDef);
            return acc.replace(styleDef, styleDefSeedIds);
        }, result);
    }
    return result;
}
